// ----------------------------------------------------------------------------
//
// Blackfin - 73322 Final test code
//
// Interrupt Service Routines
//
// Jan 13 / 03
//
// ----------------------------------------------------------------------------

#include <def21535.h>
#include <defines.h>

// ----------------------------------------------------------------------------
// Global and External definitions

.global	_EHandler;
.global _RHandler;
.global _NHandler;
.global _XHandler;
.global _HWHandler;
.global _THandler;
.global _RTCHandler;
.global _Sport1ISR;
.global	_CODEC_INIT_ISR;
.global _I9Handler;
.global _I10Handler;
.global _I11Handler;
.global _I12Handler;
.global _I13Handler;
.global _I14Handler;
.global _I15Handler;

.extern	RxBufPointer;
.extern	RxChannelEnables;
.extern	RxBuf1;
.extern	TxBufPointer;
.extern	TxChannelEnables;
.extern	TxBuf1;
.extern	Done;
.extern	Channel;
.extern	CODEC_Init_Data;


// ----------------------------------------------------------------------------
// Code
// ----------------------------------------------------------------------------
// Sport1 ISR (IVG 8)

.Section ISRCode;

_Sport1ISR:

// ----------------------------------------------------------------------------
// update the Channel number

	[--SP] = (r7:0);
	[--SP] = (P5:4);

	LoadP4(Channel)							// get current Channel
	R0.l = w[P4];
	R7 = R0;									// save for later use
	R0 += 1;									// point to next
	w[P4] = R0.l;								// save new value
	
	R1 = 0x08;
	cc = R0 == R1;								// test for roll over
	if !cc jump NoRollOver;
	
	R0 = 0x00;									// Channel has rolled over, reset to 0
	w[P4] = R0.l;

	
// ----------------------------------------------------------------------------
// get and save the CODEC data to the correct buffer

NoRollOver:
Sport1_Rx:	
	LoadR6 (RxChannelEnables)					// determine if channel is enabled
	R6 = R6 + R7;								// base + offset
	P4 = R6;
	R0 = b[P4];									// current channel enable
	cc = R0 == 0;								// off if 0, on if not 0
	if cc jump NoSave;
	
	// channel is enabled
	LoadP4 (RxBufPointer)
	R6 = w[P4];
	R5 = RxBufLength * 2;
	R4 = R5.l * R7.l (fu);						// select buffer
	R3 = R4 + R6;
	LoadR5 (RxBuf1)							// base address of Rx buffers
	R3 = R3 + R5;								// add offset to base address
	P4 = R3;
	
	LoaddP5 (SPORT1_RX)						// read current data
	R0 = w[P5];
	w[P4] = R0;									// save it.
	
NoSave:
	R6 = 0x07;
	cc = R7 == R6;
	if !cc jump Sport1_Tx;
	
	LoadP4 (RxBufPointer)
	R0.l = w[P4];
	R0 += 2;
	w[P4] = R0.l;
	R6 = RxBufLength * 2;
	cc = R0 == R6;
	if !cc jump Sport1_Tx;
	
	R0 = 0x00;
	w[P4] = R0.l;

// ----------------------------------------------------------------------------
// load and send data to the CODECs

Sport1_Tx:
	R7 +=2;
	R6 = 0x07;
	R7 = R7 & R6;
	
	LoadR6 (TxChannelEnables)
	R6 = R6 + R7;
	P4 = R6;
	R0 = b[P4];
	cc = R0 == 0x00;
	if cc jump OutputNull;						// jump if channel is OFF
	
	LoadP4 (TxBufPointer)						// channel is enabled
	R6 = w[P4];
	R5 = TxBufLength * 2;
	R4 = R5.l * R7.l(fu);
	R3 = R4 + R6;
	LoadR5 (RxBuf1)							// base address of Tx buffers
	R3 = R3 + R5;
	P4 = R3;
	
	LoaddP5 (SPORT1_TX)						// get data
	R1 = w[P4];
	w[P5] = R1; ssync;							// write to Sport1
	
TxPointerTest:									// test
	R6 = 0x06;
	cc = R7 == R6;
	if !cc jump Sport1_ISR_Exit;
	
	LoadP4 (TxBufPointer)
	R0.l = w[P4];
	R0 += 2;
	w[P4] = R0.l;
	R6 = TxBufLength * 2; // SineCosTableLength * 2;
	cc = R0 == R6;
	if !cc jump Sport1_ISR_Exit;
	
	R0 = 0x00;
	w[P4] = R0;
	
Sport1_ISR_Exit:
	LoaddP4 (SPORT1_IRQSTAT_TX)			// clear interrupts
	R0.l = 0x01;
	w[P4] = R0; ssync;
	LoaddP4 (SPORT1_IRQSTAT_RX)
	R0.l = 0x07;
	w[P4] = R0; ssync;

	LoadP4 (Done)							// show that an interrupt occured
	R0 = 0x01;
	w[P4] = R0;
	
	(P5:4) = [SP++];  						// Restore Regs
	(r7:0) = [SP++];
	rti;

OutputNull:
	R0 = 0x00;
	LoaddP4 (SPORT1_TX)
	w[P4] = R0; ssync;
	jump	TxPointerTest;
	
// ----------------------------------------------------------------------------
// This ISR is used only during CODEC initizlization and temporarily replaces
// _Sport1ISR

_CODEC_INIT_ISR:
	[--SP] = (R7:6);					// save regs
	[--SP] = (P5:3);

	LoadP3 (TxBufPointer)
	R6 = w[P3];							// read the current pointer
	P5 = R6;
	
	LoadP4 (CODEC_Init_Data)
	P4 = P4 + P5;
	R7 = w[P4];							// read the current data word
	
	LoaddP4 (SPORT1_TX)
	w[P4] = R7;
	
	R7 = CODEC_INIT_Length * 2;
	cc = R6 == R7;						// see if init is complete
	if cc jump CODEC_Init_Done;
	
	R6 += 2;							// init not complete, point to the next
	w[P3] = R6;							// word of data
	jump Tx_ISR_Done;
	
CODEC_Init_Done:
	LoadP4 (Done)						// show that CODEC init is complete
	R6 = 0x01;
	w[P4] = R6;
	
Tx_ISR_Done:
	LoaddP4 (SPORT1_IRQSTAT_TX)		// clear Tx interrupt
	R6.l = 0x07;
	w[P4] = R6; ssync;

	(P5:3) = [SP++];  					// Restore Regs
	(R7:6) = [SP++];
	rti;	

// ----------------------------------------------------------------------------

_EHandler:								// Emulation Handler 		(IVG 0)
	rte;

_RHandler:								// Reset Handler 			(IVG 1)
	rti;

_NHandler:								// NMI Handler				(IVG 2)
	rtn;

_XHandler:								// Exception Handler		(IVG 3)
	rtx;

// I4Handler							// this does not exist		(IVG 4)
						

_HWHandler:								// Hardware Error Handler	(IVG 5)
	rti;

_THandler:								// Timer Handler			(IVG 6)
	rti;

_RTCHandler:							// RTC Handler				(IVG 7)
	rti;

// I8Handler							// Sport Handler see above	(IVG 8)


_I9Handler:								// IVG 9 Handler			(IVG 9)
	rti;

_I10Handler:							// IVG 10 Handler			(IVG 10)
	rti;

_I11Handler:							// IVG 11 Handler			(IVG 11)
	rti;

_I12Handler:							// IVG 12 Handler			(IVG 12)
	rti;

_I13Handler:							// IVG 13 Handler			(IVG 13)
	rti;

_I14Handler:							// IVG 14 Handler			(IVG 14)
	rti;

_I15Handler:							// IVG 15 Handler			(IVG 15)
	rti;
